@@ -111,9 +111,15 @@ void ConfigImpl::initializeVariables()
 		{ "cupt::cache::release-file-expiration::ignore", "no" },
 		{ "cupt::console::allow-untrusted", "no" },
 		{ "cupt::console::assume-yes", "no" },
+		{ "cupt::console::actions-preview::show-archives", "no" },
+		{ "cupt::console::actions-preview::show-codenames", "no" },
+		{ "cupt::console::actions-preview::show-components", "no" },
 		{ "cupt::console::actions-preview::show-not-preferred", "for-upgrades" },
 		{ "cupt::console::actions-preview::show-details", "yes" },
+		{ "cupt::console::actions-preview::show-size-changes", "no" },
 		{ "cupt::console::actions-preview::show-summary", "yes" },
+		{ "cupt::console::actions-preview::show-vendors", "no" },
+		{ "cupt::console::actions-preview::show-versions", "no" },
 		{ "cupt::console::use-colors", "no" },
 		{ "cupt::directory", "/" },
 		{ "cupt::directory::configuration", "etc/cupt" },
@@ -157,18 +163,18 @@ void ConfigImpl::initializeVariables()
 		{ "cupt::resolver::synchronize-by-source-versions", "none" },
 		{ "cupt::resolver::track-reasons", "no" },
 		{ "cupt::resolver::type", "fair" },
-		{ "cupt::resolver::score::new", "-300" },
-		{ "cupt::resolver::score::removal", "-500" },
+		{ "cupt::resolver::score::new", "-5" },
+		{ "cupt::resolver::score::removal", "-1500" },
 		{ "cupt::resolver::score::removal-of-essential", "-200000" },
 		{ "cupt::resolver::score::upgrade", "100" },
 		{ "cupt::resolver::score::downgrade", "-800" },
-		{ "cupt::resolver::score::position-penalty", "-20" },
-		{ "cupt::resolver::score::quality-adjustment", "0" },
+		{ "cupt::resolver::score::position-penalty", "-2" },
+		{ "cupt::resolver::score::quality-adjustment", "40" },
 		{ "cupt::resolver::score::unsatisfied-recommends", "-240" },
 		{ "cupt::resolver::score::unsatisfied-suggests", "-60" },
 		{ "cupt::resolver::score::failed-synchronization", "-80" },
 		{ "cupt::worker::archives-space-limit", "0" },
-		{ "cupt::worker::defer-triggers", "no" },
+		{ "cupt::worker::defer-triggers", "auto" },
 		{ "cupt::worker::download-only", "no" },
 		{ "cupt::worker::log", "yes" },
 		{ "cupt::worker::log::levels::metadata", "1" },
